/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef ATEMU_ADC_H
#define ATEMU_ADC_H

#include "../device.h"
#include "../debug.h"

/* ADC Multiplexer Selector Register Flags */
#define ADC_MUX_REFS1 0x80
#define ADC_MUX_REFS0 0x40
#define ADC_MUX_ADLAR 0x20
#define ADC_MUX_MUX   0x1F

#define ADC_MUX_REFS      (ADC_MUX_REFS1|ADC_MUX_REFS0)
#define ADC_MUX_REFS_AREF 0
#define ADC_MUX_REFS_AVCC ADC_MUX_REFS0
#define ADC_MUX_REFS_256V ADC_MUX_REFS

/* ADC Control and Status Register Flags */
#define ADC_CONTROL_ADEN 0x80
#define ADC_CONTROL_ADSC 0x40
#define ADC_CONTROL_ADFR 0x20
#define ADC_CONTROL_ADIF 0x10
#define ADC_CONTROL_ADIE 0x08
#define ADC_CONTROL_ADPS 0x07

typedef double (*adc_sample_callback)(void *pvoid, double avcc);

typedef struct {
    device_obj    device;

    register_obj  reg_mux;
    register_obj  reg_control;
    register_obj  reg_datalow;
    register_obj  reg_datahigh;

    clock_obj     clk_tick;
    interrupt_obj int_sample;

    /* The value of the analog power supply */
    double  avcc;
    /* The value of the external analog reference */
    double  aref;
    /* Used during parameter reading */
    unsigned aref_set;

    uint8_t mux;
    uint8_t control;
    uint8_t datalow;
    uint8_t datahigh;

    adc_sample_callback sources[8];
    void* sources_dev[8];

    unsigned datalocked;
    /* Value to report at end of cycle */
    uint16_t sampleread;
    /* Raw clock from CPU Core */
    uint8_t  clockin;
    /* Bit out of the prescaler/selector combination */
    unsigned clockout;
    /* Number of clock cycles taken in this conversion */
    uint8_t  clockcount;
    /* Current prescaler value */
    uint8_t  prescaler;
    /* Times different actions take place */
    unsigned clocksample;
    uint8_t  clockcomplete;
} adc_device;

/* The analog comparator might need access to the ADC multiplexer
 * output.  This gives it a read */
double adc_read_multiplexer(adc_device* device);

/* This registers a device to answer on a particular ADC pin */
error_t adc_set_source(adc_device* device, int sourcenum,
        adc_sample_callback func, void *pvoid);
typedef error_t (*adc_set_source_func)(adc_device* device, int sourcenum,
        adc_sample_callback func, void *pvoid);

#endif /* ATEMU_TIMER8_H */
