/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#ifndef XATDB_STABS_H
#define XATDB_STABS_H

/* types from http://www.informatik.uni-frankfurt.de/doc/texi/stabs_8.html */
/* non-stab types */
#define N_UNDF          0x00
#define N_ABS           0x02
#define N_EXT           0x01
#define N_TEXT          0x04
#define N_DATA          0x06
#define N_BSS           0x08
#define N_FN_SEQ        0x0c
#define N_INDR          0x0a
#define N_COMM          0x12
#define N_SETA          0x14
#define N_SETT          0x16
#define N_SETD          0x18
#define N_SETB          0x1a
#define N_SETV          0x1c
#define N_WARNING       0x1e
#define N_FN            0x1f

/* stab types */
#define N_GSYM          0x20
#define N_FNAME         0x22
#define N_FUN           0x24
#define N_STSYM         0x26
#define N_LCSYM         0x28
#define N_MAIN          0x2a
#define N_ROSYM         0x2c
#define N_PC            0x30
#define N_NSYMS         0x32
#define N_NOMAP         0x34
#define N_OBJ           0x38
#define N_OPT           0x3c
#define N_RSYM          0x40
#define N_M2C           0x42
#define N_SLINE         0x44
#define N_DSLINE        0x46
#define N_BSLINE        0x48
#define N_BROWS         0x48
#define N_DEFD          0x4a
#define N_FLINE         0x4c
#define N_EHDECL        0x50
#define N_MOD2          0x50
#define N_CATCH         0x54
#define N_SSYM          0x60
#define N_ENDM          0x62
#define N_SO            0x64
#define N_LSYM          0x80
#define N_BINCL         0x82
#define N_SOL           0x84
#define N_PSYM          0xa0
#define N_EINCL         0xa2
#define N_ENTRY         0xa4
#define N_LBRAC         0xc0
#define N_EXCL          0xc2
#define N_SCOPE         0xc4
#define N_RBRAC         0xe0
#define N_BCOMM         0xe2
#define N_ECOMM         0xe4
#define N_ECOML         0xe8
#define N_WITH          0xea
#define N_NBTEXT        0xf0
#define N_NBDATA        0xf2
#define N_NBBSS         0xf4
#define N_NBSTS         0xf6
#define N_NBLCS         0xf8

/* Symbol Descriptors */
/* from: http://www.informatik.uni-frankfurt.de/doc/texi/stabs_9.html */

#define SD_CPLUSPLUS          ':'
#define SD_REF_REG_PARAM      'a'
#define SD_BASED_VAR          'b'
#define SD_CONSTANT           'c'
#define SD_EXCEPT_NAME        'C'
#define SD_FLOAT_REG_VAR      'd'
#define SD_FLOAT_REG_PARAM    'D'
#define SD_FILE_SCOPE_FUNC    'f'
#define SD_GLOBAL_FUNC        'F'
#define SD_GLOBAL_VAR         'G'
#define SD_INTERN_PROC        'I'
#define SD_INTERN_FUNC        'J'
#define SD_LABEL_NAME         'L'
#define SD_MODULE             'm'
#define SD_ARG_LIST_PARAM     'p'
#define SD_GNU_REG_PARAM      'P'
#define SD_STATIC_PROC        'Q'
#define SD_REG_PARAM          'R'
#define SD_REG_VAR            'r'
#define SD_FILE_SCOPE_VAR     'S'
#define SD_TYPE_NAME          't'
#define SD_STRUCT             'T'
#define SD_REF_PARAM          'v'
#define SD_PROC_SCOPE_STATIC  'V'
#define SD_CONFORMANT_ARRAY   'x'
#define SD_RETURN_VALUE       'X'

/* Type Descriptors */
/* from: http://www.informatik.uni-frankfurt.de/doc/texi/stabs_10.html */

#define TD_TYPE_REF           '('
#define TD_BUILT_IN           '-'
#define TD_CPLUSPLUS_METHOD   '#'
#define TD_POINTER            '*'
#define TD_CPLUSPLUS_REF      '&'
#define TD_TYPE_ATTR          '@'
#define TD_ARRAY              'a'
#define TD_OPEN_ARRAY         'A'
#define TD_PASCAL_SPACE       'b'
#define TD_VOLATILE_QUAL      'B'
#define TD_COMPLEX_BUILT_IN   'c'
#define TD_COBOL_PIC          'C'
#define TD_FILE               'd'
#define TD_DYMANIC_ARRAY      'D'
#define TD_ENUM               'e'
#define TD_SUB_ARRAY          'E'
#define TD_FUNC               'f'
#define TD_PASCAL_FUNC_PARAM  'F'
#define TD_COBOL_GROUP        'G'
#define TD_IMPORTED           'i'
#define TD_CONST              'k'
#define TD_COBOL_FILE         'K'
#define TD_MULTI_INST         'M'
#define TD_STRING             'n'
#define TD_STRING_PTR         'N'
#define TD_OPAQUE             'o'
#define TD_PROCEDURE          'p'
#define TD_PACKED_ARRAY       'P'
#define TD_RANGE              'r'
#define TD_FLOAT_BUILT_IN     'R'
#define TD_STRUCT             's'
#define TD_SET                'S'
#define TD_UNION              'u'
#define TD_VARIANT_REC        'v'
#define TD_WIDE_CHAR          'w'
#define TD_CROSS_REF          'x'
#define TD_ZSTRING            'z'

#endif
