/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#include <stdlib.h>
#include <stdio.h>

#include <gtk/gtk.h>

#include "toolbar.h"
#include "callbacks.h"
#include "interface.h"
#include "atemu.h"
#include "chip_data.h"
#include "chip_data_struct.h"
#include "breakpoints.h"
#include "watchpoints.h"
#include "debug.h"

static int watchpoint_chip_sel;
static int breakpoint_chip_sel;
static GtkWidget *watch_popup = NULL;
static GtkWidget *break_popup = NULL;
static int toolbar_sense;

/* functions to remove the pesky popups */

void close_watch_popup( )
{
    if ( watch_popup != NULL )
    {
        gtk_widget_destroy( watch_popup );
        watch_popup = NULL;
    }
}

void close_break_popup( )
{
    if ( break_popup != NULL )
    {
        gtk_widget_destroy( break_popup );
        break_popup = NULL;
    }
}

/* callbacks for the inevitable popups generated */

static void apply_breakpoint (GtkWidget *widget, gpointer data){
    GtkWidget *dialog;
    int line;
    int i;
    long addr;

    if( sscanf( (char *) gtk_entry_get_text( GTK_ENTRY( data ) ), 
            "%lx", &addr ) <= 0 )
    {
        dialog = gtk_message_dialog_new( GTK_WINDOW( break_popup ),
                GTK_DIALOG_DESTROY_WITH_PARENT,
                GTK_MESSAGE_ERROR,
                GTK_BUTTONS_CLOSE,
                "Invalid Address '%s'",
                (char *) gtk_entry_get_text( GTK_ENTRY( data ) ) );
        /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
        g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
                G_CALLBACK (gtk_widget_destroy),
                GTK_OBJECT (dialog));

        gtk_widget_show( dialog );
        return;
    }

    /*apply to all chips*/
    if( breakpoint_chip_sel == 0 )
    {
        for( i = 0; i < chip_array_cnt; i++ )
        {
            line = address_to_text_line( i, addr );
    
            if( line == -1 )            
            {
                dialog = gtk_message_dialog_new( GTK_WINDOW( watch_popup ),
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        GTK_MESSAGE_ERROR,
                        GTK_BUTTONS_CLOSE,
                        "Invalid Address '%s'",
                        (char *) gtk_entry_get_text( GTK_ENTRY( data ) ) );
                /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
                g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
                        G_CALLBACK (gtk_widget_destroy),
                        GTK_OBJECT (dialog));
                
                gtk_widget_show( dialog );
                
                return;
            }
            
            add_asm_breakpoint( i, addr );
        }
    }
    else /*specific chip selected*/
    {
        i = breakpoint_chip_sel - 1;
        line = address_to_text_line( i, addr );
        
        if( line == -1 )
        {
            dialog = gtk_message_dialog_new( GTK_WINDOW( watch_popup ),
                    GTK_DIALOG_DESTROY_WITH_PARENT,
                    GTK_MESSAGE_ERROR,
                    GTK_BUTTONS_CLOSE,
                    "Invalid Address '%s'",
                    (char *) gtk_entry_get_text( GTK_ENTRY( data ) ) );
            /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
            g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
                    G_CALLBACK (gtk_widget_destroy),
                    GTK_OBJECT (dialog));
            gtk_widget_show( dialog );
            return;
        }
    
        add_asm_breakpoint( i, addr );
    }
    gtk_widget_set_sensitive(menu_debug_clear_breaks, TRUE);
    close_break_popup();
}

static void change_breakpoint_chip(GtkWidget *widget, gpointer data){
    breakpoint_chip_sel = gtk_option_menu_get_history ( GTK_OPTION_MENU( data ) );
}

static void change_watchpoint_chip(GtkWidget *widget, gpointer data){
    watchpoint_chip_sel = gtk_option_menu_get_history ( GTK_OPTION_MENU( data ) );
}

static void apply_watchpoint(GtkWidget *widget, gpointer data){
    GtkWidget *dialog;
    int i;
    long addr;
    
    
    if( sscanf( (char *) gtk_entry_get_text( GTK_ENTRY( data ) ), 
            "%lx", &addr ) <= 0 )
    {
        dialog = gtk_message_dialog_new( GTK_WINDOW( watch_popup ),
                GTK_DIALOG_DESTROY_WITH_PARENT,
                GTK_MESSAGE_ERROR,
                GTK_BUTTONS_CLOSE,
                "Invalid Address '%s'",
                (char *) gtk_entry_get_text( GTK_ENTRY( data ) ) );
        /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
        g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
                G_CALLBACK (gtk_widget_destroy),
                GTK_OBJECT (dialog));

        gtk_widget_show( dialog );
        return;
    }

    /*apply to all chips*/
    if( watchpoint_chip_sel == 0 )
    {
        for( i = 0; i < chip_array_cnt; i++ )
        {
            add_watchpoint( i, addr );
        }
    }
    else /*specific chip selected*/
    {
        add_watchpoint( watchpoint_chip_sel - 1, addr );
    }
    
    close_watch_popup();
}

/* the callbacks for when the buttons get pressed */

void
button_step_clicked                    (GtkWidget *widget,
                                        gpointer   data)
{
    atemu_step();
    toolbar_sense_go();
}

void
button_cstep_clicked                      (GtkWidget *widget,
                                        gpointer   data)
{
    if(curr_chip >= 0 && curr_chip < chip_array_cnt){
        activate_auto_breakpoints(curr_chip);
        atemu_go();
        toolbar_sense_go();
    }
}

void
button_go_clicked                      (GtkWidget *widget,
                                        gpointer   data)
{
    deactivate_auto_breakpoints();
    atemu_go();
    toolbar_sense_go();
}

void
button_stop_clicked                    (GtkWidget *widget,
                                        gpointer   data)
{
    atemu_stop();
}

void
button_restart_clicked                 (GtkWidget *widget,
                                        gpointer   data)
{
    atemu_quit();

    atemu_init( NULL );
}

void
button_set_break_clicked               (GtkWidget *widget,
                                        gpointer   data)
{
    GtkWidget *chips_menu_item, *entry, *button_apply, 
        *button_close, *label1,
        *label2, *table, *hbox1, 
        *image, *label, *chips_menu, *chips_option_menu;
    char buf[128];
    int i;
    
    /*stop more than one window from opening*/
    if( break_popup != NULL )
    {
        return;
    }
    
    if( chip_array_cnt == 0 )
    {
        verb_printf( TRUE, stdout, "no chips\n");
        return;
    }
   
    breakpoint_chip_sel = curr_chip + 1;
    
    break_popup = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_title( GTK_WINDOW( break_popup ), "Set Breakpoint" );
    gtk_container_set_border_width (GTK_CONTAINER( break_popup ), 10);
    
    table = gtk_table_new( 3, 4, TRUE );
    gtk_container_add( GTK_CONTAINER( break_popup ), table );
    
    label1 = gtk_label_new( "Chip ID" );
    label2 = gtk_label_new( "Breakpoint Address" );
    gtk_table_attach_defaults( GTK_TABLE( table ), label1, 0, 2, 0, 1 );
    gtk_table_attach_defaults( GTK_TABLE( table ), label2, 2, 4, 0, 1 );

    chips_menu = gtk_menu_new( );
    chips_menu_item = gtk_menu_item_new_with_mnemonic( "All Chips" );
    gtk_menu_shell_append( GTK_MENU_SHELL( chips_menu ), chips_menu_item );
    
    for( i = 0; i < chip_array_cnt; i++ )
    {
        snprintf( buf, sizeof(buf), "Chip %d", i );
        buf[sizeof(buf)-1] = '\n';
        chips_menu_item = gtk_menu_item_new_with_mnemonic( buf );
        gtk_menu_shell_append( GTK_MENU_SHELL( chips_menu ), chips_menu_item );
    }

    chips_option_menu = gtk_option_menu_new( );
    gtk_option_menu_set_menu( GTK_OPTION_MENU( chips_option_menu ), chips_menu );
    
    gtk_option_menu_set_history( GTK_OPTION_MENU( chips_option_menu ), curr_chip + 1 );
    
    /*connect the option menu to the change_breakpoint_chip function*/
    g_signal_connect( G_OBJECT( chips_option_menu ), "changed", 
            G_CALLBACK( change_breakpoint_chip ), (gpointer) chips_option_menu );
    
    entry = gtk_entry_new( );
    /*connect the text entry to the apply_breakpoint function*/
    g_signal_connect( G_OBJECT( entry ), "activate", 
            G_CALLBACK( apply_breakpoint ), (gpointer) entry );

    gtk_table_attach_defaults( GTK_TABLE( table ), chips_option_menu, 0, 2, 1, 2 );
    gtk_table_attach_defaults( GTK_TABLE( table ), entry, 2, 4, 1, 2 );
    
    hbox1 = gtk_hbox_new( FALSE, 0 );
    image = gtk_image_new_from_stock( GTK_STOCK_NO, GTK_ICON_SIZE_BUTTON );
    label = gtk_label_new( "Add" );
    gtk_box_pack_start( GTK_BOX( hbox1 ), image, FALSE, FALSE, 3 );
    gtk_box_pack_start( GTK_BOX( hbox1 ), label, FALSE, FALSE, 3 );
    button_apply = gtk_button_new( );
    gtk_container_add( GTK_CONTAINER( button_apply ), hbox1 );
    
    /*connect the apply button to the apply_breakpoint function*/ 
    g_signal_connect( G_OBJECT( button_apply ), "clicked",
            G_CALLBACK( apply_breakpoint ), (gpointer) entry );
    gtk_table_attach_defaults( GTK_TABLE( table ), button_apply, 2, 3, 2, 3 );

    hbox1 = gtk_hbox_new( FALSE, 0 );
    button_close = gtk_button_new( );
    image = gtk_image_new_from_stock( GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON );
    label = gtk_label_new( "Cancel" );
    gtk_box_pack_start( GTK_BOX( hbox1 ), image, FALSE, FALSE, 3 );
    gtk_box_pack_start( GTK_BOX( hbox1 ), label, FALSE, FALSE, 3 );
    gtk_container_add( GTK_CONTAINER( button_close ), hbox1 );

    /* Connect the close button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT( button_close ),
            "clicked", G_CALLBACK( close_break_popup ),
            NULL );
    gtk_table_attach_defaults( GTK_TABLE( table ), button_close, 3, 4, 2, 3 );
    
    gtk_widget_show_all( break_popup );
    
    /*gtk_widget_grab_default( button_apply );*/
    gtk_widget_grab_focus( entry );
}
  
void
button_set_watch_clicked               (GtkWidget *widget,
                                        gpointer   data)
{

    GtkWidget *chips_menu_item, *entry, *button_apply, 
        *button_close, *label1,
        *label2, *table, *hbox1, 
        *image, *label, *chips_menu, *chips_option_menu;
    char buf[128];
    int i;
    
    /*avoid multiple popups*/
    if( watch_popup != NULL )
    {
        return;
    }
    
    if( chip_array_cnt == 0 )
    {
        verb_printf( TRUE, stdout, "no chips\n");
        return;
    }
   
    watchpoint_chip_sel = curr_chip + 1;
    
    watch_popup = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_title( GTK_WINDOW( watch_popup ), "Set Watchpoint" );
    gtk_container_set_border_width (GTK_CONTAINER( watch_popup ), 10);
    
    table = gtk_table_new( 3, 4, TRUE );
    gtk_container_add( GTK_CONTAINER( watch_popup ), table );
    
    label1 = gtk_label_new( "Chip ID" );
    label2 = gtk_label_new( "Watchpoint Address" );
    gtk_table_attach_defaults( GTK_TABLE( table ), label1, 0, 2, 0, 1 );
    gtk_table_attach_defaults( GTK_TABLE( table ), label2, 2, 4, 0, 1 );

    chips_menu = gtk_menu_new( );
    chips_menu_item = gtk_menu_item_new_with_mnemonic( "All Chips" );
    gtk_menu_shell_append( GTK_MENU_SHELL( chips_menu ), chips_menu_item );
    
    for( i = 0; i < chip_array_cnt; i++ )
    {
        snprintf( buf, sizeof(buf), "Chip %d", i );
        buf[sizeof(buf)-1] = '\n';
        chips_menu_item = gtk_menu_item_new_with_mnemonic( buf );
        gtk_menu_shell_append( GTK_MENU_SHELL( chips_menu ), chips_menu_item );
    }

    chips_option_menu = gtk_option_menu_new( );
    gtk_option_menu_set_menu( GTK_OPTION_MENU( chips_option_menu ), chips_menu );
    
    gtk_option_menu_set_history( GTK_OPTION_MENU( chips_option_menu ), curr_chip + 1 );
    
    /*connect the option menu to the change_watchpoint_chip function*/
    g_signal_connect( G_OBJECT( chips_option_menu ), "changed", 
            G_CALLBACK( change_watchpoint_chip ), (gpointer) chips_option_menu );
    
    entry = gtk_entry_new( );
    /*connect the text entry to the apply_watchpoint function*/
    g_signal_connect( G_OBJECT( entry ), "activate", 
            G_CALLBACK( apply_watchpoint ), (gpointer) entry );

    gtk_table_attach_defaults( GTK_TABLE( table ), chips_option_menu, 0, 2, 1, 2 );
    gtk_table_attach_defaults( GTK_TABLE( table ), entry, 2, 4, 1, 2 );
    
    hbox1 = gtk_hbox_new( FALSE, 0 );
    image = gtk_image_new_from_stock( GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_BUTTON );
    label = gtk_label_new( "Add" );
    gtk_box_pack_start( GTK_BOX( hbox1 ), image, FALSE, FALSE, 3 );
    gtk_box_pack_start( GTK_BOX( hbox1 ), label, FALSE, FALSE, 3 );
    button_apply = gtk_button_new( );
    gtk_container_add( GTK_CONTAINER( button_apply ), hbox1 );
    
    /*connect the apply button to the apply_watchpoint function*/ 
    g_signal_connect( G_OBJECT( button_apply ), "clicked",
            G_CALLBACK( apply_watchpoint ), (gpointer) entry );
    gtk_table_attach_defaults( GTK_TABLE( table ), button_apply, 2, 3, 2, 3 );

    hbox1 = gtk_hbox_new( FALSE, 0 );
    button_close = gtk_button_new( );
    image = gtk_image_new_from_stock( GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON );
    label = gtk_label_new( "Cancel" );
    gtk_box_pack_start( GTK_BOX( hbox1 ), image, FALSE, FALSE, 3 );
    gtk_box_pack_start( GTK_BOX( hbox1 ), label, FALSE, FALSE, 3 );
    gtk_container_add( GTK_CONTAINER( button_close ), hbox1 );

    /* Connect the close button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT( button_close ),
            "clicked", G_CALLBACK( close_watch_popup ),
            NULL);
    gtk_table_attach_defaults( GTK_TABLE( table ), button_close, 3, 4, 2, 3 );
    
    gtk_widget_show_all( watch_popup );
    
    /*gtk_widget_grab_default( button_apply );*/
    gtk_widget_grab_focus( entry );
}

/* toolbar sensitivity functions */

void toolbar_cstep_sense_update(){
    /* cstep is only active if we've read at least one cfile on this chip */
    
    if( !GTK_WIDGET_IS_SENSITIVE( button_cstep ) == ( toolbar_sense && curr_chip >= 0 && chip_array[curr_chip].cfile_read ) ) { 
        gtk_widget_set_sensitive( button_cstep, toolbar_sense && ( curr_chip >= 0 ) && chip_array[curr_chip].cfile_read );
        gtk_widget_set_sensitive( menu_debug_cstep, toolbar_sense && ( curr_chip >= 0 ) && chip_array[curr_chip].cfile_read );
        /* Work around a bug in GTK */
        if( GTK_WIDGET_STATE( button_cstep ) == GTK_STATE_INSENSITIVE ) {
            button_cstep->saved_state = GTK_STATE_NORMAL;
        }
    }
}

/* basic activation / deactivation of toolbar */
void toolbar_sense_set(gboolean sensitive){
    toolbar_sense = sensitive;
    gtk_widget_set_sensitive( button_step, sensitive );
    gtk_widget_set_sensitive( button_go, sensitive );
    gtk_widget_set_sensitive( button_stop, FALSE );
    gtk_widget_set_sensitive( button_restart, sensitive );
    /*gtk_widget_set_sensitive( button_set_break, sensitive );
    gtk_widget_set_sensitive( button_set_watch, sensitive );*/

    /* Prevent any instances of the pre_edit state bug */
    if( toolbar_sense == FALSE ) {
        button_step->saved_state = GTK_STATE_NORMAL;
        button_go->saved_state = GTK_STATE_NORMAL;
        button_stop->saved_state = GTK_STATE_NORMAL;
        button_restart->saved_state = GTK_STATE_NORMAL;
    }
    
    gtk_widget_set_sensitive( menu_debug_step, sensitive );
    gtk_widget_set_sensitive( menu_debug_run, sensitive );
    gtk_widget_set_sensitive( menu_debug_stop, FALSE );
    gtk_widget_set_sensitive( menu_debug_restart, sensitive );
    gtk_widget_set_sensitive( menu_debug_set_break, sensitive );
    gtk_widget_set_sensitive( menu_debug_set_watch, sensitive );

    toolbar_cstep_sense_update();
}

void toolbar_sense_go(){
    toolbar_sense_set(FALSE);
    gtk_widget_set_sensitive( button_stop, TRUE);
    gtk_widget_set_sensitive( menu_debug_stop, TRUE );
    set_sensitive_data( FALSE );
}

void toolbar_sense_stop(){
    toolbar_sense_set(TRUE);
    set_sensitive_data( TRUE );
}
