/*
* Copyright (c) 2003, University of Maryland 
* Copyright (c) 2003, Center for Satellite and Hybrid Communication Networks 
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*   * Redistributions of source code must retain the above copyright notice, 
*     this list of conditions,the authors and the following disclaimer.
*   * Redistributions in binary form must reproduce the above copyright notice,
*     this list of conditions,the authors and the following disclaimer in
*     the documentation and/or other materials provided with the distribution.
*   * Neither the name of the University of Maryland/Center for Satellite
*     and Hybrid Communication Networks nor the names of its 
*     contributors may be used to endorse or promote products derived from 
*     this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
* POSSIBILITY OF SUCH DAMAGE.
*
*/

/* Authors: 
*     Jonathan Polley (jpolley@isr.umd.edu),
*     Dionysus Blazakis (dblaze@isr.umd.edu),
*     Jonathan McGee (mcgee@isr.umd.edu),
*     Daniel Rusk (ruskd@isr.umd.edu),
*     Karl Seamon (kks@isr.umd.edu),
*     Manish Karir (karir@isr.umd.edu)
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>

#include "type_data.h"
#include "debug.h"

struct type_data *create_type_data( char *name, char *id, int bytel,
    long long int begin, long long int end )
{
    struct type_data *ret;

    ret = (struct type_data *) malloc( sizeof(struct type_data) );
    if ( ret == NULL )
    {
        return NULL;
    }

    /* Empty list */
    ret->fields = NULL;

    ret->name = strdup( name );
    ret->id = strdup( id );
    ret->byte_length = bytel;
    ret->begin = begin;
    ret->end = end;

    if (( ret->begin == 0 ) && ( ret->end == 0 ))
        ret->sign_type = 0;
    else if ( ret->begin < 0 )
        ret->sign_type = -1;
    else if ( ret->begin >= 0 )
        ret->sign_type = 1;

    return ret;
}

void add_field_to_type( struct type_data * td, struct field * f )
{
    if (( td == NULL ) || ( f == NULL ))
    {
        verb_printf( TRUE, stdout, "Passed NULL arg to add_field_to_type.\n" );
        return;
    }

    td->fields = g_slist_append( td->fields, (gpointer) f );

    /* we don't want to accidently show a struct as a dedimal value. 
       that would be really silly. */
    td->sign_type = 0;
}

int type_data_compare_id( const void * a, const void * b )
{
    struct type_data *atd = (struct type_data *) a;
    struct type_data *btd = (struct type_data *) b;

    return ( strcmp( atd->id, btd->id ) );
}

void free_type_data( void * a )
{
    struct type_data *td = (struct type_data *) a;
    GSList *curr;

    if ( td == NULL )
        return;

    for( curr = td->fields; curr != NULL; curr = curr->next )
        free_field( curr->data );

    g_slist_free( td->fields );

    free( td->name );
    free( td->id );
    free( td );
}

struct field * create_field( char * name, int bitl )
{
    struct field * ret;

    ret = (struct field *) malloc( sizeof(struct field) );
    if ( ret == NULL )
    {
        return NULL;
    }

    ret->name = strdup( name );
    ret->bit_length = bitl;

    return ret;
}

void free_field( struct field * f )
{
    if ( f )
        free( f->name );

    free( f );
}
